/*
 * MAS model :-)
 * Stator reference
 * Etat : Ialphas, Ibetas, Phialphas, Phibetas, Vitese
 *  -------------------------------------------------------------------------
 *  by Lotfi BAGHLI
 *  -------------------------------------------------------------------------
 * 24/09/1999
 */



#define S_FUNCTION_LEVEL 2
#define S_FUNCTION_NAME  mas

#define p(S)    ssGetSFcnParam(S, 0)
#define Rs(S)   ssGetSFcnParam(S, 1)
#define Rr(S)   ssGetSFcnParam(S, 2)
#define Ls(S)   ssGetSFcnParam(S, 3)
#define Lr(S)   ssGetSFcnParam(S, 4)
#define M(S)    ssGetSFcnParam(S, 5)

#define J(S)    ssGetSFcnParam(S, 6)
#define a1(S)    ssGetSFcnParam(S, 7)
#define a2(S)    ssGetSFcnParam(S, 8)
#define a3(S)    ssGetSFcnParam(S, 9)

/* #include <math.h> */
#include "simstruc.h"

#define U(element) (*uPtrs[element])  /* Pointer to Input Port0 */

const double r3=1.732050807569;
static int p;
static	double Rs, Rr, Ls, Lr, M, J, a1, a2, a3;
static	double	sigma, sigmaLs, Ts, i_Ts, Tr, i_Tr, i_Tspi_Tr;


/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 10);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 5);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 4);
    ssSetInputPortDirectFeedThrough(S, 0, 1);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 8);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, 0);
}

/* Function: mdlInitializeSampleTimes =========================================
 *    S-function is comprised of only continuous sample time elements
*/
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);

}

#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ========================================
 * Abstract:
 *    Initialize continuous states to zero
 */
static void mdlInitializeConditions(SimStruct *S)
{
/*    real_T *Rs_S    = Rs(S);*/
    real_T *x0 = ssGetContStates(S);
    int_T  i;

    for (i=0; i<5; i++) {
        *x0=0.0;
    }

/*p=mxGetPr( mexGetMatrixPtr("p"));*/
p=mxGetPr(p(S))[0];
Rs=mxGetPr(Rs(S))[0];
Rr=mxGetPr(Rr(S))[0];
Ls=mxGetPr(Ls(S))[0];
Lr=mxGetPr(Lr(S))[0];
M=mxGetPr(M(S))[0];

J=mxGetPr(J(S))[0];
a1=mxGetPr(a1(S))[0];
a2=mxGetPr(a2(S))[0];
a3=mxGetPr(a3(S))[0];

sigma=1-M*M/Ls/Lr; sigmaLs=sigma*Ls;
Ts=Ls/Rs; i_Ts=1/Ts;
Tr=Lr/Rr; i_Tr=1/Tr;
i_Tspi_Tr=i_Ts+i_Tr;
}

/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block. Generally outputs are placed in the output vector, ssGetY(S).
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T            *y    = ssGetOutputPortRealSignal(S,0);
    real_T            *x    = ssGetContStates(S);
/*    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
*/
    y[0]=x[0]; /* I alpha s */
    y[1]=x[1]; /* I beta s */
    y[2]=x[2]; /* Phi alpha s */
    y[3]=x[3]; /* Phi beta s */

    y[4]=x[0];                  /* Ias */
    y[5]=(-x[0]+x[1]*r3)/2;     /* Ibs */
    y[6]=(-x[0]-x[1]*r3)/2;     /* Ics */
    y[7]=x[4]/p;
}



#define MDL_DERIVATIVES  /* Change to #undef to remove function */
#if defined(MDL_DERIVATIVES)
  /* Function: mdlDerivatives =================================================
   * Abstract:
   *    In this function, you compute the S-function block's derivatives.
   *    The derivatives are placed in the derivative vector, ssGetdX(S).
   */
static void mdlDerivatives(SimStruct *S)
  {
double VaN, VbN, VcN, Vds, Vqs;
double Ce, Gr;

    real_T            *dx   = ssGetdX(S);
    real_T            *x    = ssGetContStates(S);
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);

/* calcul des tensions simples ( phase-neutre ) */

  VaN=(2*U(0)-U(1)-U(2))/3;
  VbN=(-U(0)+2*U(1)-U(2))/3;
  VcN=(-U(0)-U(1)+2*U(2))/3;

/* que du Clark pas de concordia */
/* calcul de Vds, Vqs; ref au stator */
  Vds=(2*VaN-VbN-VcN)/3;
  Vqs=(VbN-VcN)/r3;

dx[0]=Vds/sigmaLs-i_Tspi_Tr/sigma*x[0]-x[4]*x[1]+i_Tr/sigmaLs*x[2]+x[4]/sigmaLs*x[3];
dx[1]=Vqs/sigmaLs+x[4]*x[0]-i_Tspi_Tr/sigma*x[1]-x[4]/sigmaLs*x[2]+i_Tr/sigmaLs*x[3];
dx[2]=Vds-Rs*x[0];
dx[3]=Vqs-Rs*x[1];

/*
if (Crsgn )	Gr=a1*x[4]/p*x[4]/p+a2*x[4]/p+a3*sgn(x[4])+U(3);
	else		Gr=a1*x[4]/p*x[4]/p+a2*x[4]/p+a3+U(3);
*/
Gr=a1*x[4]/p*x[4]/p+a2*x[4]/p+a3+U(3);
Ce=1.5*p*(x[2]*x[1]-x[0]*x[3]);
dx[4]=( Ce-Gr )/J*p;

}
#endif /* MDL_DERIVATIVES */



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlInitializeConditions, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}

/*=============================*
 * Required S-function trailer *
 *=============================*/
#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
